#include    <sgl.h>
#include    <sega_sys.h>
#include    <sega_cdc.h>
#include    <sddrvs.dat>

#define     BACK_COL_ADR    (VDP2_VRAM_A1 + 0x1fffe)
#define     NBG1_CEL_ADR    (VDP2_VRAM_B0 + 0x00000)
#define     NBG1_COL_ADR    (VDP2_COLRAM  + 0x00200)
#define     SystemWork      0x060ffc00              /* System Variable Address */
#define     SystemSize      (0x06100000-0x060ffc00) /* System Variable Size */
#define     IMAGE_WIDTH     320L
#define     IMAGE_HEIGHT    240L
#define     FRAMERATE       30

void Pal2CRAM(Uint16 *Pal_Data, void *Col_Adr, Uint32 Entries);

unsigned char BkgImage[IMAGE_WIDTH * IMAGE_HEIGHT];
long    SinePos         = 0;

char    sdmap[]     =
{
    0xff,0xff,0xff,0xff
};

long	SineTable[]	=
{
	20,  24,  28,  31,  34,  37,  38,  40,
	40,  40,  39,  37,  34,  31,  28,  24,
	20,  16,  13,   9,   6,   3,   2,   0,
	 0,   0,   1,   3,   6,   9,  12,  16,
};

extern Uint32   _bstart, _bend;
extern unsigned char ChaosImage[];
extern unsigned short ChaosPal[];
extern unsigned char MaskImage[];
extern unsigned char LetterImages[12][521];

void Draw()
{
    long    OuterLoop, InnerLoop, loop;
    long    PosX;
    Uint8   *Dest   = BkgImage;
    Uint8   *Src    = ChaosImage;

    memcpy(Dest, Src, IMAGE_WIDTH * IMAGE_HEIGHT);

    PosX    = 16;

    for (loop = 0; loop < 12; loop++)
	{
		long	PosY	= 80 + SineTable[((SinePos + loop * 3) & 31)];
        Uint8    *Mask  = (MaskImage + (IMAGE_WIDTH * PosY + PosX));
        Uint8   *Dest   = (BkgImage + (IMAGE_WIDTH * PosY + PosX));
        Uint8   *Src    = LetterImages[loop];

        for (OuterLoop = 0; OuterLoop < 26; OuterLoop++)
		{
            for (InnerLoop = 0; InnerLoop < 20; InnerLoop++)
			{
				if (0 == *Mask)
				{
					if (*Src != 0)
					{
						*Dest		= *Src;
					}
				}

				Dest++;
				Src++;
				Mask++;
			}

			Dest	+= (IMAGE_WIDTH - 20);
			Mask	+= (IMAGE_WIDTH - 20);
		}

		PosX	+= 24;
	}

	SinePos++;
}

void Redraw()
{
    slBMPut(-160, -120, 159, 119, BkgImage);
}

void main(void)
{
	Uint8	*dst;
    Uint16  loop;
    CdcPly  PlayWork;
    CdcPly  *Play   = &PlayWork;

    for (dst = (Uint8 *)&_bstart; dst < (Uint8 *)&_bend; dst++)
    {
		*dst = 0;
	}

    for (dst = (Uint8 *)SystemWork, loop = 0; loop < SystemSize; loop++)
    {
		*dst = 0;
	}

    slInitSystem(TV_320x240, NULL, 1);
	slTVOff();
    slColRAMMode(CRM16_1024);
    slBack1ColSet((void *)BACK_COL_ADR, 0);

    slInitBitMap(bmNBG1, BM_512x256, (void *)NBG1_CEL_ADR);
    slBMPaletteNbg1(1);
	
    slScrPosNbg1(toFIXED(0.0), toFIXED(0.0));
    slScrAutoDisp(NBG1ON);

    Pal2CRAM(ChaosPal, (void *)NBG1_COL_ADR, 256);

    slTVOn();
    slInitSound(sddrvstsk, sizeof(sddrvstsk), sdmap, sizeof(sdmap));

    CDC_CdInit(0x00,0x00,0x05,0x0f);

    CDC_PLY_STYPE(Play) = CDC_PTYPE_TNO;
    CDC_PLY_STNO(Play)  = 2;
    CDC_PLY_SIDX(Play)  = 1;
    CDC_PLY_ETYPE(Play) = CDC_PTYPE_TNO;
    CDC_PLY_ETNO(Play)  = 99;
    CDC_PLY_EIDX(Play)  = 99;
    CDC_PLY_PMODE(Play) = CDC_PM_DFL + 10;

    CDC_CdPlay(Play);

	slCDDAOn(127,127,0,0);
	slSndVolume(127);
	
    while (1)
    {
        Draw();
        Redraw();
		slSynch();
	} 
}

void Pal2CRAM(Uint16 *Pal_Data, void *Col_Adr, Uint32 Entries)
{
    Uint16 loop;
	Uint16 *VRAM;

	VRAM = (Uint16 *)Col_Adr;

    for (loop = 0; loop < Entries; loop++)
    {
        *(VRAM++) = *(Pal_Data++);
    }
}

